<?php
/* EatingEngine E2 - A flexible, modular web framework
    Copyright (C)   2005   Thomas Gabor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

abstract class Core_Kernel
{
    //How many kernels are loaded? (only 1 or 0 should be allowed)
    static private $Count = 0; 
    static private $ReachedLevel = "";
    
    static private $RequestedKernel = "";
    static private $RequestedMode = "";
    static private $RequestedLevel = 0;
    
    private $Active = false; // Is this an active kernel?
     
    protected $Services = array();
    
    
    static public function getKernelRequest($Information)
    {
        switch ( strtolower($Information) )
        {
            case "kernel" :
            case "name" :
                return self::$RequestedKernel;
            case "mode" :
            case "mod":
                return self::$RequestedMode;
            case "level" :
            case "lvl" :
                return self::$RequestLevel;
        };
        return false;
    }

    static protected function getReachedLevel()
    {
        return self::$ReachedLevel;
    }
   
    
    
    
    final public function __construct($Mode="", $Level=0)
    {
        //we trust our Core_Main class to run the first kernel with right params
        
        //break if there already is a kernel
        if ( self::$Count > 0 )
        {
            return;
        };
        //count the actual kernel
        self::$Count += 1;
        //set the actual kernel active
        $this->Active = true;
        //check if a kernel was already run
        if ( empty(self::$RequestedKernel) )
        {
            //then this is the first kernel to be run, take over the parameters
            self::$RequestedKernel = get_class($this);
            self::$RequestedMode = $Mode;
            self::$RequestedLevel = $Level;
            self::$ReachedLevel = 0; //just 4 style
        }; //otherwise we already have our parameters...
        
        //call kernel-specific constructor if existing
        if ( method_exists($this, "construct") )
        {
            $this->construct();
        };
    }
    
    
    
    
    final public function __destruct()
    {
        //reduce the kernel counter
        self::$Count -= 1;
        //call kernel-specific destructor if existing
        if ( method_exists($this, "destruct") )
        {
            $this->destruct();
        };
    }
    
    
    
    
    final public function run($Steps="all")
    {
        //check if it is the active kernel
        if ( !$this->isActive() )
        {
            return false;
        };
        //check if we should run all levels
        if ( $Steps === "all" )
        {
            $Steps = self::$RequestedLevel + 1;        
        };
        //check if $Steps is an integer
        if ( !is_integer($Steps) )
        {
            return false;
        };
        //run the requested levels as long as they exist
        $i = 0;
        $MethodName = "";
        while ( $i < $Steps )
        {
            //check if the requested level is reached
            if ( self::$ReachedLevel == self::$RequestedLevel )
            {
                break;
            };
            //check if the needed method exists
            $MethodName = "runLevel" . (self::$ReachedLevel + 1);
            if ( method_exists($this, $MethodName) )
            {
                //call method for this level
                $this->$MethodName();
            }
            else
            {
                //we assume that we don't have any more methods
                break;
            };
            //save that the level was reached
            self::$ReachedLevel += 1;
            //increase iterator
            $i++;
        };
        //return something
        return true;
    }
   
    
    final public function isActive()
    {
        return $this->Active;
    }
   
    
    public function isServiceLoaded($Part, $ServiceName)
    {
        $ClassName = SystemTheme_Classes::getName
                    ($ServiceName, $Part, "Services");
        if ( isset($this->Services[$ClassName]) )
        {
            return true;
        };
        return false;
    }
    
    public function loadService($Part, $ServiceName)
    { 
        //check if the service is already loaded
        if ( $this->isServiceLoaded($Part, $ServiceName) )
        { 
            return 1;
        };
        $ClassName = SystemTheme_Classes::getName
                    ($ServiceName, $Part, "Services");
        //check if the service exists
        if ( !class_exists($ClassName) )
        { 
            return false;
        };
        //load the service
        $this->Services[$ClassName] = new $ClassName();
        return true;
    
    }
    
    public function getService($Part, $ServiceName)
    {
        $ClassName = SystemTheme_Classes::getName
                    ($ServiceName, $Part, "Services");
        if ( $this->isServiceLoaded($Part, $ServiceName) )
        {
            return $this->Services[$ClassName];
        };
        return false;
    }
    
    public function autogetService($Part, $ServiceName)
    {
        //check if the service has to be loaded
        if ( !$this->isServiceLoaded($Part, $ServiceName) )
        {
            //load the service
            $this->loadService($Part, $ServiceName);
        };
        //return the service
        return $this->getService($Part, $ServiceName);
    }
    
    
    public function __get($Service)
    { 
        $Params = explode("_", $Service, 2);
        //check if something has been exploded
        if ( count($Params) < 2 )
        { 
            return false;
        };
        //return the service
        return $this->autogetService($Params[0], $Params[1]);
    }
    
    
    public function __call($Service, $Params)
    {
        return $this->__get($Service);
    }
    
    
    
    
    protected function runLevel0()
    {
        //just a dummy to ensure you don't get an error by calling it
    }
    
    
   



}


 ?>